# -*- coding: utf-8 -*-
"""
Created on Tue Nov  5 14:33:06 2024

@author: mstep
"""

# https://developer.cybersource.com/docs/cybs/en-us/test-data/developer/all/so/test-data/best_practices_intro/checking_digit_scheme.html
def checksum(number):
    '''Calculate a credit card checksum.'''
    clean = number.replace(' ', '')
    reversed = clean[:-1][::-1]
    sum = 0
    double_it = True
    for ch in reversed:
        value = int(ch)
        if double_it:
            value *= 2
            if value >= 10:
                value = (value // 10) + (value % 10)
        sum += value
        double_it = not double_it
    sum = 10 - sum % 10
    if sum == 10:
        sum = 0
    return sum

def card_is_valid(number):
    '''Return True if the credit card number is valid.'''
    sum = checksum(number)
    return int(number[-1]) == sum

class bin:
    def __init__(self, brand, min_bin, max_bin):
        self.brand = brand
        self.min_bin = min_bin
        self.max_bin = max_bin

    def matches(self, number):
        '''Return True if this number matches the bin.'''
        bin_length = len(self.min_bin)
        return self.min_bin <= number[:bin_length] <= self.max_bin

def find_brand(number):
    # Example bins from https://chargebacks911.com/bank-identification-numbers/
    # There's a more complete list at https://bintable.com/card-schemes, although
    # it's hard to use and the site has many spam advertizing/download links.
    bins = [
        bin('Visa', '4', '4'),
        bin('AMEX', '34', '37'),
        bin('Diner’s Club', '36', '36'),
        bin('MasterCard', '51', '55'),
        bin('Discover Card', '6011', '6011'),
        bin('Discover Card', '622126', '622925'),
        bin('Discover Card', '644', '649'),
        bin('Discover Card', '65', '65'),
    ]
    
    for a_bin in bins:
        if a_bin.matches(number):
            return a_bin.brand
    return None


# Test credit card numbers from:
# https://www.paypalobjects.com/en_GB/vhelp/paypalmanager_help/credit_card_numbers.htm
numbers = [
    '378282246310005',
    '371449635398431',
    '378734493671000',
    '5610591081018250',
    '30569309025904',
    '38520000023237',
    '6011111111111117',
    '6011000990139424',
    '3530111333300000',
    '3566002020360505',
    '5555555555554444',
    '5105105105105100',
    '4111111111111111',
    '4012888888881881',
    '4222222222222',
    '76009244561',       # This one seems to be wrong.
    '5019717010103742',
    '6331101999990016',
]

for number in numbers:
    sum = checksum(number)
    is_valid = " valid " if card_is_valid(number) else "INVALID"
    brand = find_brand(number)
    print(f'{number:<16} ({sum}) is {is_valid}, {brand}')

